'use strict'

const { GraphQLClient } = require('graphql-request')

let buildGql = (type, root, fields, params = {}) => {
  if (fields instanceof Array && fields.length === 0) {
    return buildGqlWithoutFields(type, root, params);
  }

  return `${type}${formal(params)} {
        ${root}${invocation(params)} {
            ${typeof fields === 'string' ? fields : fields.sort().join(' ')}
        }
    }`
}

let buildGqlWithoutFields = (type, root, params = {}) =>
  `${type}${formal(params)} {
        ${root}${invocation(params)}
    }`

let formal = params => reduce(params, (a, p) => a.concat(`$${p}: ${params[p]}`))

let invocation = params => reduce(params, (a, p) => a.concat(`${p}: $${p}`))

let reduce = (params, reducer, init = []) => {
  let keys = Object.keys(params)
  if (keys.length === 0) return ''
  return `(${keys
    .sort()
    .reduce(reducer, init)
    .join(', ')})`
}

let squeeze = string => string.replace(/\s+/g, ' ')

let union = (a, b) => {
  return Array.from(new Set([...a, ...b]))
}

module.exports = (graphqlUrl, headers) => (
  type,
  root,
  paramDeclarations = {},
  defaultFields = []
) => (params = {}, fields = []) => {
  const graphQLClient = new GraphQLClient(graphqlUrl, { headers });
  const query = squeeze(
    buildGql(
      type,
      root,
      typeof fields === 'string' ? fields : union(fields, defaultFields),
      paramDeclarations
    )
  );
  return graphQLClient.request(query, params);
}
