const path = require('path');
const winston = require('winston');

const DEFAULT_LEVEL = 'debug';

const STDOUT = new winston.transports.Console({level: DEFAULT_LEVEL});

let format = (() => {
    let {format: f} = winston;
    let formatters;
    if (process.env.IS_GCP) formatters = [
        f.splat(),
        f.printf(i => `${i.level} ${i.message}`)
    ];
    else formatters = [
        f.splat(),
        f.timestamp(),
        f.colorize(),
        f.printf(i => `${i.timestamp} ${i.level} ${i.message}`)
    ];
    return f.combine(...formatters);
})();

const LOGGER = winston.createLogger({
    format,
    handleExceptions: true,
    exitOnError: false,
    transports: [STDOUT]
});

function getLevel() {
    return STDOUT.level;
}

function setLevel(level) {
    console.log(`Changing log level on console transport to ${level}`);
    STDOUT.level = level;
}

function getLogger(module) {
    let file = 'undefined';
    if (module.id) file = module.id.split(path.sep).slice(-2).join(path.sep);

    let annotate = level => (first, ...rest) => {
        let label, message, splat;

        // params are (message, ...splat)
        if (typeof first === 'string') {
            label = file;
            message = first;
            splat = rest;
        } else if (first instanceof Error) {
            label = file;
            message = `${first.message}\n${first.stack}`;
            splat = rest;
        // params are ({options}, message, ...splat)
        // the only option we consider at the moment is `label`
        } else {
            label = `${file} ${first.label}`;
            message = rest[0];
            splat = rest.slice(1);
        }

        LOGGER.log(level, `[${label}]: ${message}`, ...splat);
    };

    return Object.keys(LOGGER.levels).reduce(
        (o, l) => ({...o, [l]: annotate(l)}),
        {}
    );
}

module.exports = {
    getLevel,
    setLevel,
    getLogger
};
