const logger = require('./../libs/logger').getLogger(module)
const {wrapTimer} = require('./perfUtils')

function getQueryAsync(connector) {
    const promiseWrapper = function queryAsync(sql, options = null) {
        return new Promise((resolve, reject) => {
            connector.query(sql, options, function (err, res) {
                if (err) {
                    logger.debug('getQueryAsync(): error: %s, query: %s', err, sql)
                    return reject(err)
                }
                return resolve(res);
            });
        });
    };
    return wrapTimer(promiseWrapper, 'queryTime');
}

async function dbUpdate(server, config, configModel) {
    if (config.db.update === true) {
        // reset flag so other pods don't try to update
        configModel.update({configKey: 'db.update'}, {value: 'false'})
        const datasources = config.db.datasources
        for (let name of config.db.datasources) {
            const datasource = server.dataSources[name]
            if (datasource) {
                logger.debug(`Updating database for datasource ${name}`)
                let actual = await datasource.isActual()
                if (actual) {
                    logger.debug('Nothing to update')
                } else {
                    logger.debug('Changes detected, updating the database')
                    await datasource.autoupdate().catch((error) => {
                        logger.debug('An error occured updating the database ' + error)
                        throw error
                    })
                    logger.debug('Database update done')
                }
            }
        }
        await runUpdateSql(configModel, config.db.updateSql)
    }
}

async function runUpdateSql(db, sqls) {
    if (sqls) {
        for (let sql of sqls) {
            let results = await runTheSql(db, sql)
            logger.debug('SQL Update complete: %s', JSON.stringify(results))
        }
    }
}

async function runTheSql(db, sql) {
    logger.debug('runTheSql(): %s', sql)
    return new Promise((resolve, reject) => {
        const ds = db.dataSource
        ds.connector.query(sql, null, function (err, results) {
            if (err) {
                logger.error(err)
                reject(err)
                return
            }
            resolve(results)
        })
    })
}

module.exports = {
    getQueryAsync,
    dbUpdate
}