'use strict'

const fs = require('fs-extra')
const os = require('os')
const path = require('path')

class TempDir {
    constructor(name) {
        if (!name || name.includes(path.sep)) throw new Error('Invalid name!')
        this.prefix =
            ['cpc', process.env.NODE_ENV, name].filter(a => a).join('_') + '_'
    }

    // Idempotent: if this.path is assigned, just return it
    async create() {
        if (!this.path)
            this.path = await fs.mkdtemp(path.join(os.tmpdir(), this.prefix))
        return this.path
    }

    async destroy() {
        if (!this.path) return
        await fs.remove(this.path)
        this.path = null
    }
}

module.exports = TempDir
